@@ -1,6 +1,7 @@
 """"
 A basic representation of a 1D dataset
 """
+
 from pathlib import Path, PurePath
 import re
 import io
@@ -104,18 +105,17 @@ class Data1D:
         return self.y.size
 
     def __str__(self):
-        return "<{0}>, {1} points".format(self.name, len(self))
+        return f"<{self.name}>, {len(self)} points"
 
     def __repr__(self):
         msk = self._mask
         if np.all(self._mask):
             msk = None
 
-        d = {"filename": str(self.filename), "msk": msk, "data": self.data}
         if self.filename is not None:
-            return "Data1D(data={filename!r}," " mask={msk!r})".format(**d)
+            return f"Data1D(data={str(self.filename)!r}, mask={msk!r})"
         else:
-            return "Data1D(data={data!r}," " mask={msk!r})".format(**d)
+            return f"Data1D(data={self.data!r}, mask={msk!r})"
 
     @property
     def x(self):
@@ -488,6 +488,12 @@ class Data1D:
         else:
             fname = f
 
+        if isinstance(fname, PurePath):
+            # use a PurePath, not a system specific path type
+            # because Posix systems can't deal with WindowsPath
+            # and vice versa. This becomes an issue when pickling.
+            fname = PurePath(fname)
+
         self.filename = fname
         self.name = Path(fname).stem
 
@@ -581,7 +587,7 @@ class Data1D:
         """
         if self._y_err is None:
             raise RuntimeError(
-                "Can't synthesise new dataset without y_err" "uncertainties"
+                "Can't synthesise new dataset without y_err uncertainties"
             )
 
         rng = check_random_state(random_state)
