@@ -66,7 +66,7 @@ class ReflectReduce:
             self.reflect_klass = SpatzNexus
         else:
             raise ValueError(
-                "Instrument prefix not known. Must be one of" " ['PLP']"
+                "Instrument prefix not known. Must be one of ['PLP', 'SPZ']"
             )
 
         if isinstance(direct, ReflectNexus):
@@ -340,9 +340,9 @@ class ReflectReduce:
         reduction["m_lambda"] = self.reflected_beam.m_lambda
         reduction["nspectra"] = self.n_spectra
         reduction["start_time"] = self.reflected_beam.start_time
-        reduction[
-            "datafile_number"
-        ] = self.datafile_number = self.reflected_beam.datafile_number
+        reduction["datafile_number"] = self.datafile_number = (
+            self.reflected_beam.datafile_number
+        )
         reduction["direct_beam"] = self.direct_beam
         reduction["reflected_beam"] = self.reflected_beam
 
@@ -438,28 +438,24 @@ class ReflectReduce:
         self.y_err /= scale
 
     def write_offspecular(self, f, scanpoint=0):
-        d = dict()
-        d["time"] = strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime())
-        d["_rnumber"] = self.reflected_beam.datafile_number
-        d["_numpointsz"] = np.size(self.m_ref, 1)
-        d["_numpointsy"] = np.size(self.m_ref, 2)
+        """
+        Writes reduced offspecular data to a file
 
-        s = string.Template(_template_ref_xml)
+        Parameters
+        ----------
+        f : {str, filehandle}
+            Uses `np.savez` to save data. When loading these files you can use
+            `npzfile = np.load(outfile); m_qz = npzfile['m_qz']`
 
-        # filename = 'off_PLP{:07d}_{:d}.xml'.format(self._rnumber, index)
-        d["_r"] = repr(self.m_ref[scanpoint].tolist()).strip(",[]")
-        d["_qz"] = repr(self.m_qz[scanpoint].tolist()).strip(",[]")
-        d["_dr"] = repr(self.m_ref_err[scanpoint].tolist()).strip(",[]")
-        d["_qx"] = repr(self.m_qx[scanpoint].tolist()).strip(",[]")
-
-        thefile = s.safe_substitute(d)
-
-        with possibly_open_file(f, "wb") as g:
-            if "b" in g.mode:
-                thefile = thefile.encode("utf-8")
-
-            g.write(thefile)
-            g.truncate()
+        scanpoint : int
+        """
+        np.savez(
+            f,
+            m_qz=self.m_qz[scanpoint],
+            m_qx=self.m_qx[scanpoint],
+            m_ref=self.m_ref[scanpoint],
+            m_ref_err=self.m_ref_err[scanpoint],
+        )
 
     def _create_metadata_header(self):
         header = []
