@@ -23,11 +23,13 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THIS SOFTWARE.
 
 """
+
 from contextlib import contextmanager
 import time
 from functools import lru_cache
 import numbers
 import warnings
+from enum import Enum
 
 import numpy as np
 import scipy
@@ -40,6 +42,14 @@ from refnx.analysis import (
     possibly_create_parameter,
     Transform,
 )
+from refnx.util import general
+
+try:
+    from refnx.reflect._creflect import gepore
+except ImportError:
+
+    def gepore(*args, **kwds):
+        raise RuntimeError("gepore is not available")
 
 
 # some definitions for resolution smearing
@@ -115,6 +125,13 @@ def available_backends():
     except ImportError:
         pass
 
+    try:
+        from refnx.reflect import _numba_reflect as _nr
+
+        backends.append("numba_parratt")
+    except ImportError:
+        pass
+
     # try:
     #     import jax as jax
     #     from jax.config import config
@@ -142,10 +159,13 @@ def get_reflect_backend(backend="c"):
 
     Parameters
     ----------
-    backend: {'python', 'cython', 'c', 'pyopencl', 'py_parratt', 'c_parratt'}
+    backend: {'python', 'cython', 'c', 'pyopencl', 'py_parratt', 'c_parratt',
+              'numba_parratt', 'abeles_vectorised'}
         The module that calculates the reflectivity. Speed should go in the
-        order: c > pyopencl / cython > py_parratt > python. If a particular
-        method is not available the function falls back:
+        order:
+        numba_parratt > c_parratt > c > pyopencl / cython > py_parratt > python.
+
+        If a particular method is not available the function falls back:
         cython/pyopencl --> c --> --> python.
         c_parratt --> py_parratt.
 
@@ -161,6 +181,10 @@ def get_reflect_backend(backend="c"):
     be installed. It may not as accurate as the other options. 'pyopencl' is
     only included for completeness. The 'pyopencl' backend is also harder to
     use with multiprocessing-based parallelism.
+    'abeles_vectorised' is vectorised such that many slab sets can be
+    calculated simultaneously. As such, it requires a 3-dimensional array for
+    specification of the slabs. It is not a 1:1 replacement for the other
+    kernels.
     """
     backend = backend.lower()
 
@@ -191,6 +215,15 @@ def get_reflect_backend(backend="c"):
         except ImportError:
             warnings.warn("Can't use the cython abeles backend")
             return get_reflect_backend("c")
+    elif backend == "abeles_vectorised":
+        try:
+            from refnx.reflect._cyreflect import abeles_vectorised
+
+            return abeles_vectorised
+        except ImportError:
+            raise ValueError(
+                "Can't use the abeles_vectorised backend, it's not available"
+            )
     elif backend == "c":
         try:
             from refnx.reflect import _creflect as _c
@@ -199,6 +232,16 @@ def get_reflect_backend(backend="c"):
         except ImportError:
             warnings.warn("Can't use the C abeles backend")
             return get_reflect_backend("python")
+    elif backend == "numba_parratt":
+        try:
+            from refnx.reflect._numba_reflect import numba_parratt
+
+            return numba_parratt
+        except ImportError:
+            warnings.warn(
+                "Can't use the numba_parratt backend, requires numba be installed"
+            )
+            return get_reflect_backend("c_parratt")
     elif backend == "py_parratt":
         from refnx.reflect._reflect import parratt
 
@@ -250,10 +293,12 @@ def use_reflect_backend(backend="c"):
 
     Parameters
     ----------
-    backend: {'python', 'cython', 'c', 'pyopencl'}, str
+    backend: {'python', 'cython', 'c', 'pyopencl', 'py_parratt', 'c_parratt',
+              'numba_parratt', 'abeles_vectorised'}, str
         The function that calculates the reflectivity. Speed should go in the
-        order: c > pyopencl / cython > python. If a particular method is not
-        available the function falls back: cython/pyopencl --> c --> python.
+        order: numba_parratt > c_parratt > c > pyopencl / cython > python. If a
+        particular method is not available the function falls back to another
+        option.
 
     Yields
     ------
@@ -262,11 +307,16 @@ def use_reflect_backend(backend="c"):
 
     Notes
     -----
-    'c' is preferred for most circumstances.
+    'c' is preferred for most circumstances, because it's pretty much
+    guaranteed to be present.
     'pyopencl' uses a GPU to calculate reflectivity and requires that pyopencl
     be installed. It may not as accurate as the other options. 'pyopencl' is
     only included for completeness. The 'pyopencl' backend is also harder to
     use with multiprocessing-based parallelism.
+    'abeles_vectorised' is vectorised such that many slab sets can be
+    calculated simultaneously. As such, it requires a 3-dimensional array for
+    specification of the slabs. It is not a 1:1 replacement for the other
+    kernels.
     """
     global kernel
     f = kernel
@@ -275,6 +325,20 @@ def use_reflect_backend(backend="c"):
     kernel = f
 
 
+class SpinChannel(Enum):
+    """
+    Describes the incident and scattered spin state of a polarised neutron beam.
+    """
+
+    UP_UP = (1, 1)
+    UP_DOWN = (1, 0)
+    DOWN_UP = (0, 1)
+    DOWN_DOWN = (0, 0)
+
+    def __repr__(self):
+        return f"SpinChannel.{self.name}"
+
+
 class ReflectModel:
     r"""
     Parameters
@@ -299,6 +363,11 @@ class ReflectModel:
 
         This value is turned into a Parameter during the construction of this
         object.
+        Constant dq/q resolution smearing is deactivated if
+        `dq_type` is set to `'pointwise'` AND point-by-point resolution
+        information is supplied to either the
+        :meth:`refnx.reflect.ReflectModel.__call__` or
+        :meth:`refnx.reflect.ReflectModel.model` methods.
     threads: int, optional
         Specifies the number of threads for parallel calculation. This
         option is only applicable if you are using the ``_creflect``
@@ -306,25 +375,28 @@ class ReflectModel:
         ``_reflect``. If `threads == -1` then all available processors are
         used.
     quad_order: int, optional
-        the order of the Gaussian quadrature polynomial for doing the
+        the order of the Gaussian quadrature polynomial for doing pointwise
         resolution smearing. default = 17. Don't choose less than 13. If
         quad_order == 'ultimate' then adaptive quadrature is used. Adaptive
         quadrature will always work, but takes a _long_ time (2 or 3 orders
         of magnitude longer). Fixed quadrature will always take a lot less
-        time. BUT it won't necessarily work across all samples. For
+        time, BUT it won't necessarily work across all samples. For
         example, 13 points may be fine for a thin layer, but will be
         atrocious at describing a multilayer with bragg peaks.
+        If `dq_type='constant'` then this value is ignored.
     dq_type: {'pointwise', 'constant'}, optional
         Chooses whether pointwise or constant dQ/Q resolution smearing (see
         `dq` keyword) is used. To use pointwise smearing the `x_err` keyword
-        provided to `Objective.model` method must be an array, otherwise the
-        smearing falls back to 'constant'.
+        provided to :meth:`refnx.reflect.ReflectModel.model` must be an array,
+        otherwise the smearing falls back to 'constant'.
     q_offset: float or refnx.analysis.Parameter, optional
         Compensates for uncertainties in the angle at which the measurement is
         performed. A positive/negative `q_offset` corresponds to a situation
         where the measured q values (incident angle) may have been under/over
         estimated, and has the effect of shifting the calculated model to
         lower/higher effective q values.
+    spin: refnx.reflect.SpinChannel
+        The polarisation channel that is being calculated.
     """
 
     def __init__(
@@ -374,13 +446,21 @@ class ReflectModel:
             Units = Angstrom**-1
         p : refnx.analysis.Parameters, optional
             parameters required to calculate the model
-        x_err : np.ndarray
-            dq resolution smearing values for the dataset being considered.
+        x_err : {np.ndarray, float} optional
+            Specifies how the instrumental resolution smearing is carried out
+            for each of the points in `x`.
+            See :func:`refnx.reflect.reflectivity` for further details.
 
         Returns
         -------
         reflectivity : np.ndarray
             Calculated reflectivity
+
+        Notes
+        -----
+        If `x_err` is not provided then the calculation will fall back to
+        the constant dq/q smearing specified by the `dq` attribute of this
+        object.
         """
         return self.model(x, p=p, x_err=x_err)
 
@@ -463,14 +543,21 @@ class ReflectModel:
             Units = Angstrom**-1
         p : refnx.analysis.Parameters, optional
             parameters required to calculate the model
-        x_err : np.ndarray
-            dq resolution smearing values for the dataset being considered.
+        x_err : {np.ndarray, float} optional
+            Specifies how the instrumental resolution smearing is carried out
+            for each of the points in `x`.
+            See :func:`refnx.reflect.reflectivity` for further details.
 
         Returns
         -------
         reflectivity : np.ndarray
             Calculated reflectivity
 
+        Notes
+        -----
+        If `x_err` is not provided then the calculation will fall back to
+        the constant dq/q smearing specified by the `dq` attribute of this
+        object.
         """
         if p is not None:
             self.parameters.pvals = np.array(p)
@@ -478,9 +565,11 @@ class ReflectModel:
             # fallback to what this object was constructed with
             x_err = float(self.dq)
 
+        slabs = self.structure.slabs()[:, :4]
+
         return reflectivity(
             x,
-            self.structure.slabs()[..., :4],
+            slabs,
             scale=self.scale.value,
             bkg=self.bkg.value,
             dq=x_err,
@@ -532,6 +621,325 @@ class ReflectModel:
         return self._parameters
 
 
+class PolarisedReflectModel(ReflectModel):
+    """
+    Extension of ReflectModel for polarised neutron reflectometry.
+    See `refnx.reflect.ReflectModel` for documentation of arguments.
+
+    Parameters
+    ----------
+    spin: refnx.reflect.SpinChannel
+        Specifies the spin state the model is associated with.
+    Aguide: float
+        Angle of applied field. This value should be 270 or 90 degrees for
+        the applied field to lie in the plane of the sample, perpendicular to
+        the beam propagation direction. For a magnetic moment to be parallel
+        or anti-parallel to the applied field `thetaM` should be 90 or -90 deg
+        respectively.
+    """
+
+    def __init__(
+        self,
+        structure,
+        scale=1,
+        bkg=1e-7,
+        name="",
+        dq=5.0,
+        threads=-1,
+        quad_order=17,
+        dq_type="pointwise",
+        q_offset=0,
+        spin=None,
+        Aguide=270,
+    ):
+        super().__init__(
+            structure,
+            name=name,
+            scale=scale,
+            bkg=bkg,
+            threads=threads,
+            quad_order=quad_order,
+            dq=dq,
+            dq_type=dq_type,
+            q_offset=q_offset,
+        )
+        self.spin = spin
+        self.Aguide = Aguide
+
+    def __repr__(self):
+        return (
+            f"PolarisedReflectModel({self._structure!r}, name={self.name!r},"
+            f" scale={self.scale!r}, bkg={self.bkg!r},"
+            f" dq={self.dq!r}, threads={self.threads},"
+            f" quad_order={self.quad_order!r}, dq_type={self.dq_type!r},"
+            f" q_offset={self.q_offset!r}, spin={self.spin!r}, Aguide={self.Aguide!r})"
+        )
+
+    def model(self, x, p=None, x_err=None):
+        r"""
+        Calculate the reflectivity of this model
+
+        Parameters
+        ----------
+        x : float or np.ndarray
+            q values for the calculation.
+            Units = Angstrom**-1
+        p : refnx.analysis.Parameters, optional
+            parameters required to calculate the model
+        x_err : {np.ndarray, float} optional
+            Specifies how the instrumental resolution smearing is carried out
+            for each of the points in `x`.
+            See :func:`refnx.reflect.reflectivity` for further details.
+
+        Returns
+        -------
+        reflectivity : np.ndarray
+            Calculated reflectivity
+
+        Notes
+        -----
+        If `x_err` is not provided then the calculation will fall back to
+        the constant dq/q smearing specified by the `dq` attribute of this
+        object.
+        """
+        if p is not None:
+            self.parameters.pvals = np.array(p)
+        if x_err is None or self.dq_type == "constant":
+            # fallback to what this object was constructed with
+            x_err = float(self.dq)
+
+        slabs = self.structure.slabs()
+        spin = self.spin
+        if slabs.shape[1] == 5:
+            # unpolarised for some reason
+            slabs = slabs[..., :4]
+            spin = None
+        if slabs.shape[1] == 7:
+            # polarised
+            slabs = np.take_along_axis(
+                slabs, np.array([0, 1, 2, 3, 5, 6])[None, :], axis=1
+            )
+
+        return reflectivity(
+            x,
+            slabs,
+            scale=self.scale.value,
+            bkg=self.bkg.value,
+            dq=x_err,
+            threads=self.threads,
+            quad_order=self.quad_order,
+            q_offset=self.q_offset,
+            spin=spin,
+            Aguide=self.Aguide,
+        )
+
+
+class ReflectModelTL(ReflectModel):
+    r"""
+    Calculates reflectivity using angle-of-incidence/wavelength.
+
+    In most circumstances the use of `ReflectModel` is preferred, which works
+    with momentum transfer. However, for some wavelength dispersive experiments
+    the scattering length density profile (SLD) can be wavelength dependent.
+    This is more common for X-ray reflectometry than with neutrons.
+    This class deals with wavelength dependent SLD profiles. It will be slower
+    than using `ReflectModel`. There is no point in using this class if there
+    are no wavelength dependent Scatterers contained within the system.
+
+    Parameters
+    ----------
+    structure : refnx.reflect.Structure
+        The interfacial structure. Should contain at least one wavelength
+        dependent `Scatterer`, otherwise use of `ReflectModel` is preferred.
+    scale : float or refnx.analysis.Parameter, optional
+        scale factor. All model values are multiplied by this value before
+        the background is added. This is turned into a Parameter during the
+        construction of this object.
+    bkg : float or refnx.analysis.Parameter, optional
+        Q-independent constant background added to all model values. This is
+        turned into a Parameter during the construction of this object.
+    name : str, optional
+        Name of the Model
+    dq : float or refnx.analysis.Parameter, optional
+
+        - `dq == 0` then no resolution smearing is employed.
+        - `dq` is a float or refnx.analysis.Parameter
+           a constant dQ/Q resolution smearing is employed.  For 5% resolution
+           smearing supply 5.
+
+        This value is turned into a Parameter during the construction of this
+        object.
+    threads: int, optional
+        Specifies the number of threads for parallel calculation. This
+        option is only applicable if you are using the ``_creflect``
+        module. The option is ignored if using the pure python calculator,
+        ``_reflect``. If `threads == -1` then all available processors are
+        used.
+    quad_order: int, optional
+        the order of the Gaussian quadrature polynomial for doing the
+        resolution smearing. default = 17. Don't choose less than 13. If
+        quad_order == 'ultimate' then adaptive quadrature is used. Adaptive
+        quadrature will always work, but takes a _long_ time (2 or 3 orders
+        of magnitude longer). Fixed quadrature will always take a lot less
+        time. BUT it won't necessarily work across all samples. For
+        example, 13 points may be fine for a thin layer, but will be
+        atrocious at describing a multilayer with bragg peaks.
+    dq_type: {'pointwise', 'constant'}, optional
+        Chooses whether pointwise or constant dQ/Q resolution smearing (see
+        `dq` keyword) is used. To use pointwise smearing the `x_err` keyword
+        provided to `Objective.model` method must be an array, otherwise the
+        smearing falls back to 'constant'.
+    t_offset: float or refnx.analysis.Parameter, optional
+        Compensates for uncertainties in the angle at which the measurement is
+        performed. A positive/negative `t_offset` corresponds to a situation
+        where the measured t values (incident angle) may have been under/over
+        estimated, and has the effect of shifting the calculated model to
+        lower/higher effective q values.
+
+    Example
+    -------
+
+    >>> from refnx.reflect import MaterialSLD, SLD, ReflectModelTL
+    >>> air = SLD(0)
+    >>> si = SLD(2.07)
+    >>> gd2o3 = MaterialSLD("Gd2O3", 7.41)  # wavelength dependent Scatterer
+    >>> s = air | gd2o3(50, 3) | si
+    >>> model = ReflectModelTL(s)
+    >>> t = [0.65] * 60    # angle of incidence
+    >>> l = np.geomspace(2, 20, 60)  # wavelength
+    >>> model(np.c_[t, l])   # calculate reflectivity
+    """
+
+    def __init__(
+        self,
+        structure,
+        scale=1,
+        bkg=1e-7,
+        name="",
+        dq=5.0,
+        threads=-1,
+        quad_order=17,
+        dq_type="pointwise",
+        t_offset=0,
+    ):
+        self._t_offset = possibly_create_parameter(t_offset)
+        super().__init__(
+            structure,
+            name=name,
+            scale=scale,
+            bkg=bkg,
+            threads=threads,
+            quad_order=quad_order,
+            dq=dq,
+            dq_type=dq_type,
+        )
+        delattr(self, "_q_offset")
+        self._q_offset = None
+
+    def model(self, x, p=None, x_err=None):
+        r"""
+        Calculate the reflectivity of this model
+
+        Parameters
+        ----------
+        x : (float, float) or np.ndarray
+            angle of incidence/wavelength values for the calculation.
+            If an array `x` should have shape `(N, 2)`, where the first column
+            of the `N` datapoints corresponds to angle of incidence (degrees)
+            and the second column corresponds to their wavelength (angstrom**-1).
+        p : refnx.analysis.Parameters, optional
+            parameters required to calculate the model
+        x_err : np.ndarray
+            dq resolution smearing values for the dataset being considered.
+
+        Returns
+        -------
+        reflectivity : np.ndarray
+            Calculated reflectivity
+
+        """
+        if p is not None:
+            self.parameters.pvals = np.array(p)
+
+        # x has to be (N, 2) x[:, 0] is the AOI, x[:, 1] is wavelength
+        tl = np.atleast_2d(np.asarray(x))
+        ls = tl[:, 1]
+        slabs = []
+        original_wavelength = self.structure.wavelength
+
+        try:
+            for lam in ls:
+                self.structure.wavelength = lam
+                slabs.append(self.structure.slabs()[..., :4])
+        finally:
+            self.structure.wavelength = original_wavelength
+
+        q = general.q(tl[:, 0] + self.t_offset.value, tl[:, 1])
+
+        if x_err is None or self.dq_type == "constant":
+            # fallback to what this object was constructed with
+            x_err = q * float(self.dq) / 100.0
+
+        qo = self.quad_order
+
+        unique_slabs, idxs, inverse_idxs = np.unique(
+            slabs, axis=0, return_index=True, return_inverse=True
+        )
+        msks = [np.squeeze(inverse_idxs) == idx for idx in idxs]
+        R = np.empty_like(q)
+        for msk, a_unique_slabs in zip(msks, unique_slabs):
+            if np.count_nonzero(x_err[msk]):
+                # point-wise resolution smearing
+                R[msk] = _smeared_kernel_pointwise(
+                    q[msk],
+                    a_unique_slabs,
+                    x_err[msk],
+                    quad_order=qo,
+                    threads=1,
+                )
+            else:
+                # no resolution smearing
+                R[msk] = abeles(q[msk], a_unique_slabs, threads=1)
+
+        R *= self.scale.value
+        R += self.bkg.value
+        return np.squeeze(R)
+
+    @property
+    def q_offset(self):
+        pass
+
+    @property
+    def t_offset(self):
+        r"""
+        :class:`refnx.analysis.Parameter` - compensates for any angular
+        misalignment during an experiment.
+        """
+        return self._t_offset
+
+    @t_offset.setter
+    def t_offset(self, value):
+        self._t_offset.value = value
+
+    @property
+    def structure(self):
+        r"""
+        :class:`refnx.reflect.Structure` - object describing the interface of
+        a reflectometry sample.
+
+        """
+        return self._structure
+
+    @structure.setter
+    def structure(self, structure):
+        self._structure = structure
+        p = Parameters(name="instrument parameters")
+        p.extend([self.scale, self.bkg, self.dq, self.t_offset])
+
+        self._parameters = Parameters(name=self.name)
+        self._parameters.extend([p, structure.parameters])
+
+
 def reflectivity(
     q,
     slabs,
@@ -541,6 +949,8 @@ def reflectivity(
     quad_order=17,
     threads=-1,
     q_offset=0,
+    spin=None,
+    Aguide=270,
 ):
     r"""
     Abeles/Parratt formalism for calculating reflectivity from a stratified
@@ -554,7 +964,8 @@ def reflectivity(
         Units = Angstrom**-1
     slabs : np.ndarray
         coefficients required for the calculation, has shape (2 + N, 4),
-        where N is the number of layers
+        where N is the number of layers. For magnetic systems the number of
+        columns the shape will be (2 + N, 6).
 
         - slabs[0, 0]
            ignored
@@ -584,6 +995,17 @@ def reflectivity(
         - slabs[-1, 3]
            roughness between backing and layer N
 
+        If the system is magnetic then there are two extra columns:
+
+        - slab[N, 4]
+            Magnetic SLD correction (/1e-6 Angstrom**-2)
+        - slab[N, 5]
+            Angle of magnetic moment in plane (degrees). See
+            https://github.com/reflectivity/analysis/tree/master/validation
+            for details.
+
+        Note that this slab representation is slightly different to that returned by
+        `refnx.reflect.Structure.slabs()`.
     scale : float
         scale factor. All model values are multiplied by this value before
         the background is added
@@ -626,6 +1048,10 @@ def reflectivity(
         where the measured q values (incident angle) may have been under/over
         estimated, and has the effect of shifting the calculated model to
         lower/higher effective q values.
+    spin: refnx.reflect.SpinChannel
+        The polarisation channel that is being calculated.
+    Aguide: float
+        Angle of applied field.
 
     Example
     -------
@@ -639,12 +1065,21 @@ def reflectivity(
     >>> print(reflectivity(q, slabs))
 
     """
+    q = np.asarray(q)
+
     # cast q_offset to float, if it's a Parameter
     q_offset = float(q_offset)
 
+    if slabs.shape[1] == 4:
+        fkernel = kernel
+    elif slabs.shape[1] == 6:
+        fkernel = _gepore_wrapper(spin, Aguide)
+    else:
+        raise ValueError(f"slabs are wrong, {slabs.shape=}")
+
     # constant dq/q smearing
     if isinstance(dq, numbers.Real) and float(dq) == 0:
-        return kernel(
+        return fkernel(
             q + q_offset, slabs, scale=scale, bkg=bkg, threads=threads
         )
     elif isinstance(dq, numbers.Real):
@@ -652,7 +1087,7 @@ def reflectivity(
         return (
             scale
             * _smeared_kernel_constant(
-                q + q_offset, slabs, dq, threads=threads
+                q + q_offset, slabs, dq, threads=threads, fkernel=fkernel
             )
         ) + bkg
 
@@ -666,7 +1101,11 @@ def reflectivity(
             smeared_rvals = (
                 scale
                 * _smeared_kernel_adaptive(
-                    qvals_flat + q_offset, slabs, dqvals_flat, threads=threads
+                    qvals_flat + q_offset,
+                    slabs,
+                    dqvals_flat,
+                    threads=threads,
+                    fkernel=fkernel,
                 )
                 + bkg
             )
@@ -681,6 +1120,7 @@ def reflectivity(
                     dqvals_flat,
                     quad_order=quad_order,
                     threads=threads,
+                    fkernel=fkernel,
                 )
                 + bkg
             )
@@ -694,7 +1134,7 @@ def reflectivity(
     ):
         qvals_for_res = dq[:, 0, :] + q_offset
         # work out the reflectivity at the kernel evaluation points
-        smeared_rvals = kernel(qvals_for_res, slabs, threads=threads)
+        smeared_rvals = fkernel(qvals_for_res, slabs, threads=threads)
 
         # multiply by probability
         smeared_rvals *= dq[:, 1, :]
@@ -723,7 +1163,26 @@ def gauss_legendre(n):
     return scipy.special.p_roots(n)
 
 
-def _smear_kernel(x, w, q, dq, threads):
+def _gepore_wrapper(spin, Aguide):
+    _c = {
+        SpinChannel.UP_UP: 0,
+        SpinChannel.UP_DOWN: 1,
+        SpinChannel.DOWN_UP: 2,
+        SpinChannel.DOWN_DOWN: 3,
+    }
+
+    def wrapped_fun(q, w, *args, **kwds):
+        kwds["Aguide"] = Aguide
+        arr = gepore(q, w, *args, **kwds)[_c[spin]]
+        return arr.reshape(q.shape)
+
+    if spin not in _c.keys():
+        raise ValueError("spin must be an enum from refnx.reflect.SpinChannel")
+
+    return wrapped_fun
+
+
+def _smear_kernel(x, w, q, dq, threads, fkernel=kernel):
     """
     Adaptive Gaussian quadrature integration
 
@@ -748,10 +1207,10 @@ def _smear_kernel(x, w, q, dq, threads):
     prefactor = 1 / np.sqrt(2 * np.pi)
     gauss = prefactor * np.exp(-0.5 * x * x)
     localq = q + x * dq / _FWHM
-    return kernel(localq, w, threads=threads) * gauss
+    return fkernel(localq, w, threads=threads) * gauss
 
 
-def _smeared_kernel_adaptive(qvals, w, dqvals, threads=-1):
+def _smeared_kernel_adaptive(qvals, w, dqvals, threads=-1, fkernel=kernel):
     """
     Resolution smearing that uses adaptive Gaussian quadrature integration
     for the convolution.
@@ -783,20 +1242,21 @@ def _smeared_kernel_adaptive(qvals, w, dqvals, threads=-1):
     smeared_rvals = np.zeros(qvals.size)
     warnings.simplefilter("ignore", Warning)
     for idx, val in enumerate(qvals):
-        smeared_rvals[idx], err = scipy.integrate.quadrature(
+        smeared_rvals[idx], err = scipy.integrate.quad(
             _smear_kernel,
             -_INTLIMIT,
             _INTLIMIT,
-            tol=2 * np.finfo(np.float64).eps,
-            rtol=2 * np.finfo(np.float64).eps,
-            args=(w, qvals[idx], dqvals[idx], threads),
+            epsabs=0.0,
+            args=(w, qvals[idx], dqvals[idx], threads, fkernel),
         )
 
     warnings.resetwarnings()
     return smeared_rvals
 
 
-def _smeared_kernel_pointwise(qvals, w, dqvals, quad_order=17, threads=-1):
+def _smeared_kernel_pointwise(
+    qvals, w, dqvals, quad_order=17, threads=-1, fkernel=kernel
+):
     """
     Resolution smearing that uses fixed order Gaussian quadrature integration
     for the convolution.
@@ -845,19 +1305,19 @@ def _smeared_kernel_pointwise(qvals, w, dqvals, quad_order=17, threads=-1):
     va = qvals - _INTLIMIT * dqvals / _FWHM
     vb = qvals + _INTLIMIT * dqvals / _FWHM
 
-    va = va[:, np.newaxis]
-    vb = vb[:, np.newaxis]
+    va = va[..., np.newaxis]
+    vb = vb[..., np.newaxis]
 
-    qvals_for_res = (np.atleast_2d(abscissa) * (vb - va) + vb + va) / 2.0
-    smeared_rvals = kernel(qvals_for_res, w, threads=threads)
+    qvals_for_res = (abscissa[np.newaxis, :] * (vb - va) + vb + va) / 2.0
+    smeared_rvals = fkernel(qvals_for_res, w, threads=threads)
 
-    smeared_rvals = np.reshape(smeared_rvals, (qvals.size, abscissa.size))
+    # smeared_rvals = np.reshape(smeared_rvals, (qvals.size, abscissa.size))
 
     smeared_rvals *= np.atleast_2d(gaussvals * weights)
-    return np.sum(smeared_rvals, 1) * _INTLIMIT
+    return np.sum(smeared_rvals, -1) * _INTLIMIT
 
 
-def _smeared_kernel_constant(q, w, resolution, threads=-1):
+def _smeared_kernel_constant(q, w, resolution, threads=-1, fkernel=kernel):
     """
     Fast resolution smearing for constant dQ/Q.
 
@@ -881,7 +1341,7 @@ def _smeared_kernel_constant(q, w, resolution, threads=-1):
     """
 
     if resolution < 0.5:
-        return kernel(q, w, threads=threads)
+        return fkernel(q, w, threads=threads)
 
     resolution /= 100
     gaussnum = 51
@@ -911,7 +1371,7 @@ def _smeared_kernel_constant(q, w, resolution, threads=-1):
     gauss_x = _cached_linspace(-1.7 * resolution, 1.7 * resolution, gaussnum)
     gauss_y = gauss(gauss_x, resolution / _FWHM)
 
-    rvals = kernel(xlin, w, threads=threads)
+    rvals = fkernel(xlin, w, threads=threads)
     smeared_rvals = np.convolve(rvals, gauss_y, mode="same")
     smeared_rvals *= gauss_x[1] - gauss_x[0]
 
@@ -1020,7 +1480,7 @@ class MixedReflectModel:
             tscales = scales
         elif scales is not None and len(structures) != len(scales):
             raise ValueError(
-                "You need to supply scale factor for each" " structure"
+                "You need to supply scale factor for each structure"
             )
         else:
             tscales = [1 / len(structures)] * len(structures)
@@ -1256,8 +1716,11 @@ class FresnelTransform(Transform):
 
     def __repr__(self):
         # use repr for sld's because they could be `reflect.SLD` objects
-        return "FresnelTransform({0}, {1}, dq={2})".format(
-            repr(self.sld_fronting), repr(self.sld_backing), repr(self.dq)
+        return (
+            f"FresnelTransform("
+            f"{self.sld_fronting!r}, "
+            f"{self.sld_backing!r}, "
+            f"dq={self.dq!r})"
         )
 
     def __call__(self, x, y, y_err=None, x_err=0):
